# Copyright (C) 2010 A. S. Budden
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Custom URL Scheme - Create custom URL schemes for shortening URLS

Custom URL Scheme is a simple plugin for Bazaar, providing a means of
branching from remote servers without having to type out the full address
of the server.   For example, if it is desirable to retrieve branches stored
on a server with sftp, you could shorten:

    bzr branch sftp://my.username@server.com/path/to/bazaar/projects/work/myproj

to:

    bzr branch serv:work/myproj

In normal use, there are no required commands, but some commands are available 
to simplify creation of url schemes:

 * custom-url-scheme-add - Add a new URL scheme
 * custom-url-scheme-list - List existing URL schemes
 * custom-url-scheme-remove - Remove an existing URL scheme

To get started, create a new URL scheme (for the example above) with:

    bzr custom-url-scheme-add serv sftp://my.username@server.com/path/to/bazaar/projects/

Then check it worked with:

    bzr custom-url-scheme-list

Very little error checking is carried out at present.
"""
    
version_info = (1,0,0)

from bzrlib.directory_service import directories
from bzrlib.config import GlobalConfig

def _register_directory(name):
    directories.register_lazy(name + ':', 'bzrlib.plugins.custom_url_schemes.scheme_directory',
            'SchemeDirectory',
            'Custom URLScheme directory service',)

def GetCustomSchemes():
    config = GlobalConfig()
    parser = config._get_parser()
    stored_custom_schemes = parser.get('Custom URL Schemes', {})
    custom_schemes = {}
    for name, command in stored_custom_schemes.items():
        while name.endswith(':'):
            name = name[:-1]
        custom_schemes[name] = command
    return custom_schemes

def register_directories():
    custom_schemes = GetCustomSchemes()
    for name in custom_schemes.keys():
        _register_directory(name)

register_directories()

from bzrlib.commands import plugin_cmds
lazy_commands = (
    # module, command, [aliases]
    ('bzrlib.plugins.custom_url_schemes.commands', 'cmd_custom_url_scheme_add', []),
    ('bzrlib.plugins.custom_url_schemes.commands', 'cmd_custom_url_scheme_list', []),
    ('bzrlib.plugins.custom_url_schemes.commands', 'cmd_custom_url_scheme_remove', []),
)

for module, name, aliases in lazy_commands:
    plugin_cmds.register_lazy(name, aliases, module)
